@@ -1,14 +1,18 @@
 import logging
+import traceback
 from logging.handlers import RotatingFileHandler
 from io import BytesIO
 from os.path import abspath, join, dirname
 from typing import Optional, Callable, Literal
 
+import atap_corpus
+from atap_corpus._types import TCorpora
 from atap_corpus.corpus.corpus import DataFrameCorpus
 from pandas import DataFrame
 from panel.widgets import Tqdm
 
 from atap_corpus_loader.controller.CorpusExportService import CorpusExportService
+from atap_corpus_loader.controller.events import EventType, EventManager
 from atap_corpus_loader.controller.loader_service import LoaderService
 from atap_corpus_loader.controller.loader_service.FileLoadError import FileLoadError
 from atap_corpus_loader.controller.loader_service.FileLoaderService import FileLoaderService
@@ -24,7 +28,7 @@ class Controller:
     """
     Provides methods for indirection between the corpus loading logic and the user interface
     Holds a reference to the latest corpus built.
-    The build_callback_fn will be called when a corpus is built (can be set using set_build_callback()).
+    The callbacks will be called when a corpus is built (can be set using set_build_callback()).
     """
 
     def __init__(self, root_directory: str):
@@ -45,7 +49,7 @@ class Controller:
 
         self.corpora: UniqueNameCorpora = UniqueNameCorpora()
 
-        self.build_callback_fn: Optional[Callable] = None
+        self.event_manager: EventManager = EventManager(self.LOGGER)
 
         self.build_tqdm = Tqdm(visible=False)
         self.export_tqdm = Tqdm(visible=False)
@@ -63,14 +67,9 @@ class Controller:
         file_handler.setLevel(logging.DEBUG)
         file_handler.setFormatter(formatter)
 
-        console_handler = logging.StreamHandler()
-        console_handler.setLevel(logging.ERROR)
-        console_handler.setFormatter(formatter)
-
         Controller.LOGGER = logging.getLogger(__name__)
         Controller.LOGGER.setLevel(logging.DEBUG)
         Controller.LOGGER.addHandler(file_handler)
-        Controller.LOGGER.addHandler(console_handler)
 
         Controller.LOGGER.info('Logger started')
 
@@ -82,10 +81,8 @@ class Controller:
         Controller.LOGGER.info(f"Success displayed: {success_msg}")
         self.notifier_service.notify_success(success_msg)
 
-    def set_build_callback(self, callback: Callable):
-        if not callable(callback):
-            raise ValueError("Provided callback function must be a callable")
-        self.build_callback_fn = callback
+    def register_event_callback(self, event_type: EventType, callback: Callable):
+        self.event_manager.register_event_callback(event_type, callback)
 
     def get_latest_corpus(self) -> Optional[DataFrameCorpus]:
         if len(self.corpora) == 0:
@@ -103,6 +100,9 @@ class Controller:
 
         return corpora_dict
 
+    def get_mutable_corpora(self) -> TCorpora:
+        return self.corpora
+
     def set_loader_service_type(self, loader_type: Literal['file', 'oni']):
         if loader_type == 'file':
             self.loader_service = self.file_loader_service
@@ -118,11 +118,13 @@ class Controller:
             self.loader_service.add_corpus_files(filepath_ls, include_hidden, self.build_tqdm)
             self.corpus_headers = self.loader_service.get_inferred_corpus_headers()
         except FileLoadError as e:
+            self.LOGGER.error(traceback.format_exc())
             self.display_error(str(e))
             self.unload_all()
             self.build_tqdm.visible = False
             return False
 
+        self.event_manager.trigger_callbacks(EventType.LOAD)
         self.build_tqdm.visible = False
         return True
 
@@ -133,11 +135,13 @@ class Controller:
             self.loader_service.add_meta_files(filepath_ls, include_hidden, self.build_tqdm)
             self.meta_headers = self.loader_service.get_inferred_meta_headers()
         except FileLoadError as e:
+            self.LOGGER.error(traceback.format_exc())
             self.display_error(str(e))
             self.unload_all()
             self.build_tqdm.visible = False
             return False
 
+        self.event_manager.trigger_callbacks(EventType.LOAD)
         self.build_tqdm.visible = False
         return True
 
@@ -176,15 +180,15 @@ class Controller:
             return False
 
         try:
-            if self.build_callback_fn is not None:
-                self.build_callback_fn(corpus)
-                Controller.LOGGER.debug(f"build_corpus method: callback function called")
+            corpus.add_dtm(atap_corpus.parts.dtm.DTM.from_docs_with_vectoriser(corpus.docs()), 'tokens')
         except Exception as e:
-            Controller.LOGGER.exception("Exception while calling build callback: ")
-            self.display_error(f"Build callback error: {e}")
+            Controller.LOGGER.exception("Exception while building DTM: ")
+            self.display_error(str(e))
             self.build_tqdm.visible = False
             return False
 
+        self.event_manager.trigger_callbacks(EventType.BUILD, corpus)
+
         self.build_tqdm.visible = False
         Controller.LOGGER.debug(f"build_corpus method: corpus building complete")
 
@@ -196,6 +200,10 @@ class Controller:
         for corpus in reversed(self.corpora.items()):
             corpus_df: DataFrame = corpus.to_dataframe()
 
+            parent_name: Optional[str] = None
+            if corpus.parent is not None:
+                parent_name = corpus.parent.name
+
             name: Optional[str] = corpus.name
             num_rows: int = len(corpus)
             headers: list[str] = []
@@ -212,12 +220,13 @@ class Controller:
             if not corpus_df.empty:
                 first_row_data = [str(x) for x in corpus_df.iloc[0]]
 
-            corpora_info.append(ViewCorpusInfo(name, num_rows, headers, dtypes, first_row_data))
+            corpora_info.append(ViewCorpusInfo(name, num_rows, parent_name, headers, dtypes, first_row_data))
 
         return corpora_info
 
     def delete_corpus(self, corpus_name: str):
         self.corpora.remove(corpus_name)
+        self.event_manager.trigger_callbacks(EventType.DELETE)
 
     def rename_corpus(self, corpus_name: str, new_name: str):
         Controller.LOGGER.debug(f"Renaming corpus named '{corpus_name}' to '{new_name}'")
@@ -233,6 +242,8 @@ class Controller:
         except Exception as e:
             self.display_error(f"Unexpected error while renaming: {e}")
 
+        self.event_manager.trigger_callbacks(EventType.RENAME, corpus)
+
     def get_loaded_file_counts(self) -> dict[str, int]:
         corpus_file_set = self.loader_service.get_loaded_corpus_files()
         meta_file_set = self.loader_service.get_loaded_meta_files()
@@ -261,6 +272,8 @@ class Controller:
             self.meta_headers = []
             self.meta_link_header = None
 
+        self.event_manager.trigger_callbacks(EventType.UNLOAD)
+
     def unload_all(self):
         Controller.LOGGER.debug("All files unloaded")
         self.loader_service.remove_all_files()
@@ -271,6 +284,8 @@ class Controller:
         self.corpus_link_header = None
         self.meta_link_header = None
 
+        self.event_manager.trigger_callbacks(EventType.UNLOAD)
+
     def get_loaded_corpus_files(self) -> set[FileReference]:
         return self.loader_service.get_loaded_corpus_files()
 
